@@ -34,12 +34,14 @@ using Mono.Unix;
 using Hyena;
 using Hyena.Collections;
 using Mtp;
+using MTP = Mtp;
 
 using Banshee.Base;
 using Banshee.Dap;
 using Banshee.ServiceStack;
 using Banshee.Library;
 using Banshee.Sources;
+using Banshee.Playlist;
 using Banshee.Configuration;
 using Banshee.Collection;
 using Banshee.Collection.Database;
@@ -104,7 +106,7 @@ namespace Banshee.Dap.Mtp
             } else {
                 string mtp_serial = devices[0].SerialNumber;
                 if (!String.IsNullOrEmpty (mtp_serial)) {
-                    if (mtp_serial.Contains (device.Uuid)) {
+                    if (mtp_serial.Contains (device.Serial)) {
                         mtp_device = devices[0];
                         mtp_source = this;
                     }
@@ -138,6 +140,10 @@ namespace Banshee.Dap.Mtp
                 }
             }
             AcceptableMimeTypes = mimetypes.ToArray ();
+
+            AddDapProperty (Catalog.GetString ("Serial number"), mtp_device.SerialNumber);
+            AddDapProperty (Catalog.GetString ("Version"), mtp_device.Version);
+            AddDapProperty (Catalog.GetString ("Battery level"), String.Format ("{0:0%}", mtp_device.BatteryLevel/100.0));
         }
 
         protected override void LoadFromDevice ()
@@ -146,6 +152,8 @@ namespace Banshee.Dap.Mtp
             try {
                 List<Track> files = mtp_device.GetAllTracks (delegate (ulong current, ulong total, IntPtr data) {
                     //user_event.Progress = (double)current / total;
+                    // Translators: {0} is the name of the MTP audio device (eg Gabe's Zen Player), {1} is the
+                    // current track being loaded, and {2} is the total # of tracks that will be loaded.
                     SetStatus (String.Format (Catalog.GetString ("Loading {0} - {1} of {2}"), Name, current, total), false);
                     return 0;
                 });
@@ -160,15 +168,31 @@ namespace Banshee.Dap.Mtp
                     if ((track_id = DatabaseTrackInfo.GetTrackIdForUri (MtpTrackInfo.GetPathFromMtpTrack (mtp_track), source_ids)) > 0) {
                         track_map[track_id] = mtp_track;
                     } else {
-                        MtpTrackInfo track = new MtpTrackInfo (mtp_track);
+                        MtpTrackInfo track = new MtpTrackInfo (mtp_device, mtp_track);
                         track.PrimarySource = this;
                         track.Save (false);
                         track_map[track.TrackId] = mtp_track;
                     }
                 }
+
+                Hyena.Data.Sqlite.HyenaSqliteCommand insert_cmd = new Hyena.Data.Sqlite.HyenaSqliteCommand (
+                    @"INSERT INTO CorePlaylistEntries (PlaylistID, TrackID)
+                        SELECT ?, TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND ExternalID = ?");
+                foreach (MTP.Playlist playlist in mtp_device.GetPlaylists ()) {
+                    PlaylistSource pl_src = new PlaylistSource (playlist.Name, this);
+                    pl_src.Save ();
+                    // TODO a transaction would make sense here (when the threading issue is fixed)
+                    foreach (int id in playlist.TrackIds) {
+                        ServiceManager.DbConnection.Execute (insert_cmd, pl_src.DbId, this.DbId, id);
+                    }
+                    pl_src.UpdateCounts ();
+                    AddChildSource (pl_src);
+                }
+
             } catch (Exception e) {
                 Log.Exception (e);
             }
+            OnTracksAdded ();
         }
 
         public override void Import ()
@@ -189,28 +213,54 @@ namespace Banshee.Dap.Mtp
             }
         }
 
+        public override void SyncPlaylists ()
+        {
+            lock (mtp_device) {
+                List<MTP.Playlist> device_playlists = new List<MTP.Playlist> (mtp_device.GetPlaylists ());
+                foreach (MTP.Playlist playlist in device_playlists) {
+                    playlist.Remove ();
+                }
+                device_playlists.Clear ();
+    
+                // Add playlists from Banshee to the device
+                foreach (Source child in Children) {
+                    PlaylistSource from = child as PlaylistSource;
+                    if (from != null && from.Count > 0) {
+                        MTP.Playlist playlist = new MTP.Playlist (mtp_device, from.Name);
+                        foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (String.Format (
+                            "SELECT CoreTracks.ExternalID FROM CoreTracks{0} WHERE {1}",
+                            from.DatabaseTrackModel.JoinFragment, from.DatabaseTrackModel.Condition)))
+                        {
+                            playlist.AddTrack (track_id);
+                        }
+                        playlist.Save ();
+                    }
+                }
+            }
+        }
+
+
         public override bool CanRename {
             get { return !(IsAdding || IsDeleting); }
         }
 
+        private static SafeUri empty_file = new SafeUri (Paths.Combine (Paths.ApplicationCache, "mtp.mp3"));
         protected override void OnTracksDeleted ()
         {
             // Hack to get the disk usage indicate to be accurate, which seems to
             // only be updated when tracks are added, not removed.
-            SafeUri empty_file = new SafeUri (Paths.Combine (Paths.ApplicationCache, "mtp.mp3"));
             try {
-                using (System.IO.TextWriter writer = new System.IO.StreamWriter (Banshee.IO.File.OpenWrite (empty_file, true))) {
-                    writer.Write ("foo");
-                }
-                Track mtp_track = new Track (System.IO.Path.GetFileName (empty_file.LocalPath), 3);
                 lock (mtp_device) {
+                    using (System.IO.TextWriter writer = new System.IO.StreamWriter (Banshee.IO.File.OpenWrite (empty_file, true))) {
+                        writer.Write ("foo");
+                    }
+                    Track mtp_track = new Track (System.IO.Path.GetFileName (empty_file.LocalPath), 3);
+
                     mtp_device.UploadTrack (empty_file.AbsolutePath, mtp_track, mtp_device.MusicFolder);
                     mtp_device.Remove (mtp_track);
+                    Banshee.IO.File.Delete (empty_file);
                 }
-            } finally {
-                Banshee.IO.File.Delete (empty_file);
-            }
-
+            } catch {}
             base.OnTracksDeleted ();
         }
 
@@ -224,25 +274,25 @@ namespace Banshee.Dap.Mtp
 
         public override long BytesUsed {
             get {
-				long count = 0;
+                long count = 0;
                 lock (mtp_device) {
                     foreach (DeviceStorage s in mtp_device.GetStorage ()) {
                         count += (long) s.MaxCapacity - (long) s.FreeSpaceInBytes;
                     }
                 }
-				return count;
+                return count;
             }
         }
         
         public override long BytesCapacity {
             get {
-				long count = 0;
+                long count = 0;
                 lock (mtp_device) {
                     foreach (DeviceStorage s in mtp_device.GetStorage ()) {
                         count += (long) s.MaxCapacity;
                     }
                 }
-				return count;
+                return count;
             }
         }
 
@@ -257,9 +307,8 @@ namespace Banshee.Dap.Mtp
 
             Track mtp_track = TrackInfoToMtpTrack (track, fromUri);
             bool video = (track.MediaAttributes & TrackMediaAttributes.VideoStream) != 0;
-            Console.WriteLine ("Sending file {0}, is video? {1}", fromUri.LocalPath, video);
             lock (mtp_device) {
-                mtp_device.UploadTrack (fromUri.LocalPath, mtp_track, video ? mtp_device.VideoFolder : mtp_device.MusicFolder, OnUploadProgress);
+                mtp_device.UploadTrack (fromUri.LocalPath, mtp_track, GetFolderForTrack (track), OnUploadProgress);
             }
 
             // Add/update album art
@@ -271,13 +320,13 @@ namespace Banshee.Dap.Mtp
 
                     if (supports_jpegs && can_sync) {
                         try {
-                            Gdk.Pixbuf pic = ServiceManager.Get<Banshee.Collection.Gui.ArtworkManager> ().LookupScale (
-                                CoverArtSpec.CreateArtistAlbumId (track.ArtistName, track.AlbumTitle), thumb_width
+                            Gdk.Pixbuf pic = ServiceManager.Get<Banshee.Collection.Gui.ArtworkManager> ().LookupScalePixbuf (
+                                track.ArtworkId, thumb_width
                             );
                             if (pic != null) {
                                 byte [] bytes = pic.SaveToBuffer ("jpeg");
                                 album.Save (bytes, (uint)pic.Width, (uint)pic.Height);
-                                pic.Dispose ();
+                                Banshee.Collection.Gui.ArtworkManager.DisposePixbuf (pic);
                             }
                             album_cache[key] = album;
                         } catch {}
@@ -289,12 +338,22 @@ namespace Banshee.Dap.Mtp
                 }
             }
 
-            MtpTrackInfo new_track = new MtpTrackInfo (mtp_track);
+            MtpTrackInfo new_track = new MtpTrackInfo (mtp_device, mtp_track);
             new_track.PrimarySource = this;
             new_track.Save (false);
             track_map[new_track.TrackId] = mtp_track;
         }
 
+        private Folder GetFolderForTrack (TrackInfo track)
+        {
+            if (track.HasAttribute (TrackMediaAttributes.Podcast)) {
+                return mtp_device.PodcastFolder;
+            } else if (track.HasAttribute (TrackMediaAttributes.VideoStream)) {
+                return mtp_device.VideoFolder;
+            } else {
+                return mtp_device.MusicFolder;
+            }
+        }
 
         private int OnUploadProgress (ulong sent, ulong total, IntPtr data)
         {
@@ -316,7 +375,7 @@ namespace Banshee.Dap.Mtp
                 if (album_cache.ContainsKey (key)) {
                     Album album = album_cache[key];
                     album.RemoveTrack (mtp_track);
-                    if (album.TrackCount == 0) {
+                    if (album.Count == 0) {
                         album.Remove ();
                         album_cache.Remove (key);
                     }
@@ -327,21 +386,18 @@ namespace Banshee.Dap.Mtp
         public Track TrackInfoToMtpTrack (TrackInfo track, SafeUri fromUri)
         {
 			Track f = new Track (System.IO.Path.GetFileName (fromUri.LocalPath), (ulong) Banshee.IO.File.GetSize (fromUri));
-			f.Album = track.AlbumTitle;
-			f.Artist = track.ArtistName;
-			f.Duration = (uint)track.Duration.TotalMilliseconds;
-			f.Genre = track.Genre;
-			f.Rating = (ushort)(track.Rating * 20);
-			f.Title = track.TrackTitle;
-			f.TrackNumber = (ushort)track.TrackNumber;
-			f.UseCount = (uint)track.PlayCount;
-            f.ReleaseDate = track.Year + "0101T0000.0";
+            MtpTrackInfo.ToMtpTrack (track, f);
             return f;
-		}
+        }
 
+        private bool disposed = false;
         public override void Dispose ()
         {
-			base.Dispose ();
+            if (disposed)
+                return;
+
+            disposed = true;
+            base.Dispose ();
 
             if (mtp_device != null) {
                 lock (mtp_device) {
@@ -350,13 +406,13 @@ namespace Banshee.Dap.Mtp
             }
 
             ServiceManager.SourceManager.RemoveSource (this);
-
             mtp_device = null;
             mtp_source = null;
         }
 
         protected override void Eject ()
         {
+            base.Eject ();
             Dispose ();
         }
 
